#include <SDL.h>
#include <SDL_image.h>
#include "SDL_ttf.h"

#include "jalbButtonArray.h"

struct buttonArray *initButtonArray(int x, int y, int width, int fontW, int fontH, int buttH, int edgeW, int cols, int rows) {
    struct buttonArray *ba = malloc(sizeof(struct buttonArray));

    ba->x = x;
    ba->y = y;

    ba->width = width;
    ba->height = (edgeW + buttH) * rows;

    ba->fontW = fontW;
    ba->fontH = fontH;

    ba->buttH = buttH;

    ba->edgeW = edgeW;
    ba->cols = cols;
    ba->rows = rows;

    ba->textures = calloc(cols * rows, sizeof(SDL_Texture*));
    ba->textureWs = malloc(cols * rows * sizeof(int));

    return ba;
}

void freeButtonArray(struct buttonArray *buttAr) {
    int i;
    for(i = 0; i < buttAr->cols * buttAr->rows; i++) {
        if(buttAr->textures[i] != NULL) {
            SDL_DestroyTexture(buttAr->textures[i]);
        }
    }
    free(buttAr->textures);
    free(buttAr->textureWs);
    free(buttAr);
}

void buttArSetText(struct buttonArray *ba, SDL_Renderer *renderer, char *str, int buttX, int buttY, SDL_Color *color, TTF_Font *font) {
    int strLen = 0;
    while(str[strLen] != '\0') {
        strLen++;
    }

    SDL_Surface *txt = TTF_RenderText_Blended(font, str, *color);
    SDL_Texture *tx = SDL_CreateTextureFromSurface(renderer, txt);

    SDL_FreeSurface(txt);

    int index = buttY * ba->cols + buttX;

    if(ba->textures[index] != NULL) {
        SDL_DestroyTexture(ba->textures[index]);
    }
    ba->textures[index] = tx;
    ba->textureWs[index] = strLen;
}

void buttArRender(struct buttonArray *ba, SDL_Renderer *renderer) {
    int buttonWidth = (ba->width - ba->edgeW * (ba->cols + 1)) / ba->cols;

    SDL_Rect rect;
    rect.h = ba->buttH;
    rect.w = buttonWidth;

    int iy;
    for(iy = 0; iy < ba->rows; iy++) {
        int ix;
        for(ix = 0; ix < ba->cols; ix++) {
            // Render x
            rect.x = ba->x + ba->edgeW + (ba->edgeW + buttonWidth) * ix;
            rect.y = ba->y + ba->edgeW + (ba->edgeW + ba->buttH) * iy;

            SDL_SetRenderDrawColor(renderer, 100, 100, 100, 1);
            SDL_RenderFillRect(renderer, &rect);

            SDL_Rect srcrect;
            srcrect.x = 0;
            srcrect.y = 0;
            srcrect.w = ba->textureWs[iy * ba->cols + ix] * ba->fontW;
            srcrect.h = ba->fontH;

            SDL_Rect dstrect;
            dstrect.x = rect.x + (rect.w - ba->textureWs[iy * ba->cols + ix] * ba->fontW) / 2;
            dstrect.y = rect.y + (rect.h - ba->fontH) / 2;
            dstrect.w = srcrect.w;
            dstrect.h = srcrect.h;

            SDL_RenderCopy(renderer, ba->textures[iy * ba->cols + ix], &srcrect, &dstrect);
        }
    }
}

void buttArRenderCursor(struct buttonArray *ba, SDL_Renderer *renderer, int buttSelected, int cursorIndex, int cursorW, int cursorH) {
    int buttonWidth = (ba->width - ba->edgeW * (ba->cols + 1)) / ba->cols;

    SDL_Rect rect;
    rect.h = ba->buttH;
    rect.w = buttonWidth;

    int iy;
    for(iy = 0; iy < ba->rows; iy++) {
        int ix;
        for(ix = 0; ix < ba->cols; ix++) {
            // Render x
            rect.x = ba->x + ba->edgeW + (ba->edgeW + buttonWidth) * ix;
            rect.y = ba->y + ba->edgeW + (ba->edgeW + ba->buttH) * iy;

            SDL_SetRenderDrawColor(renderer, 100, 100, 100, 1);
            SDL_RenderFillRect(renderer, &rect);

            SDL_Rect srcrect;
            srcrect.x = 0;
            srcrect.y = 0;
            srcrect.w = ba->textureWs[iy * ba->cols + ix] * ba->fontW;
            srcrect.h = ba->fontH;

            SDL_Rect dstrect;
            dstrect.x = rect.x + (rect.w - ba->textureWs[iy * ba->cols + ix] * ba->fontW) / 2;
            dstrect.y = rect.y + (rect.h - ba->fontH) / 2;
            dstrect.w = srcrect.w;
            dstrect.h = srcrect.h;

            SDL_RenderCopy(renderer, ba->textures[iy * ba->cols + ix], &srcrect, &dstrect);

//            printf("buttSelected: %d\n", buttSelected);
            if(buttSelected == 0) {
                // Draw a cursor in this button.
                dstrect.x += cursorIndex * ba->fontW;
                dstrect.y += (ba->fontH -cursorH) / 2;
                dstrect.w = cursorW;
                dstrect.h = cursorH;

                SDL_SetRenderDrawColor(renderer, 0, 0, 0, 1);
                SDL_RenderFillRect(renderer, &dstrect);
            }

            buttSelected--;
        }
    }
}

int buttArClick(struct buttonArray *ba, int mx, int my) {
    int buttonWidth = (ba->width - ba->edgeW * (ba->cols + 1)) / ba->cols;

    mx -= ba->x;
    my -= ba->y;

//    printf("relMouse (%d, %d)\n", mx, my);

    int ySect = my / (ba->buttH + ba->edgeW);
    my -= ySect * (ba->buttH + ba->edgeW);

    int xSect = mx / (buttonWidth + ba->edgeW);
    mx -= xSect * (buttonWidth + ba->edgeW);

    if(mx > ba->edgeW) {
        if(xSect < ba->cols) {
            if(my > ba->edgeW) {
                if(ySect < ba->rows) {
                    return ySect * ba->cols + xSect;
                }
            }
        }
    }
    return -1;
}

void buttArFree(struct buttonArray *buttAr){
    int i;
    int numButts = buttAr->cols * buttAr->rows;
    for(i = 0; i < numButts; i++) {
        if(buttAr->textures[i]) {
            SDL_DestroyTexture(buttAr->textures[i]);
        }
    }
    free(buttAr->textures);
    free(buttAr->textureWs);

    free(buttAr);
}
